/*
 * Last updated: 2011/12/10
 * ~Philip Peng
 * 
 * This file is part of BomberNome.
 * 
 * BomberNome is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * BomberNome is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with BomberNome.	If not, see <http://www.gnu.org/licenses/>.
 */

#include "bombernome.h"

// Private variables
static int led_grid[LED_WIDTH][LED_HEIGHT];
static int monome_leds_max[4];
static byte *monome_1x, *monome_2x, *monome_3x, *monome_4x;
static byte *monome_1y, *monome_2y, *monome_3y, *monome_4y;

// Called by main, init hardware
void setup_display() {
	
	// Output ports
	DDRA = 0xFF;	// PORTA[8] is for the x pins of Monome #1 and #2
	DDRB = 0xFF;	// PORTB[8] is for the y pins of Monome #1 and #2
	DDRAD = 0xFF;	// PORTAD[8] is for the x pins of Monome #3 and #4
		
	// Port T needs a few other thing set
	DDRT = 0xFF;	// PTT[8] is for the y pins of Monome #3 and #4
	MODRR = 0x00;	// Don't use Module Routing Register
	PWME = 0x00;	// Don't use PWM
		
	// Malloc lists (not enough static memory?)
	monome_1x = (byte *)malloc((MONOME_PINS * MONOME_PINS) * sizeof(byte));
	monome_2x = (byte *)malloc((MONOME_PINS * MONOME_PINS) * sizeof(byte));
	monome_3x = (byte *)malloc((MONOME_PINS * MONOME_PINS) * sizeof(byte));
	monome_4x = (byte *)malloc((MONOME_PINS * MONOME_PINS) * sizeof(byte));
	monome_1y = (byte *)malloc((MONOME_PINS * MONOME_PINS) * sizeof(byte));
	monome_2y = (byte *)malloc((MONOME_PINS * MONOME_PINS) * sizeof(byte));
	monome_3y = (byte *)malloc((MONOME_PINS * MONOME_PINS) * sizeof(byte));
	monome_4y = (byte *)malloc((MONOME_PINS * MONOME_PINS) * sizeof(byte));
}

// Called by update_game, update monome display
void update_led_grid() {

#ifdef TEST_LED
	// For testing
	byte x, y;
	static int xtest = 0, ytest = 0;
	for (y = 0; y < LED_HEIGHT; y++) {
		for (x = 0; x < LED_WIDTH; x++) {
			led_grid[x][y] = 0;
		}
	}
	led_grid[xtest][ytest] = 1;
	xtest++;
	if (xtest >= LED_WIDTH) {
		xtest = 0;
		ytest++;
		if (ytest >= LED_HEIGHT) {
			ytest = 0;
		}
	}
#else
	
	byte x, y;
	
	// Loop through entire grid and set led_grid
	// We assume led_grid is at least double the width and height of game_grid
	// Note: everything is offset by one since we want a nice border
	for (y = 0; y < GRID_HEIGHT; y++) {
		for (x = 0; x < GRID_WIDTH; x++) {	
			switch(game_grid[x][y]) {
				case EMPTY:			// --/--
					led_grid[2*x+1][2*y+1] = 0;
					led_grid[2*x+2][2*y+1] = 0;
					led_grid[2*x+1][2*y+2] = 0;
					led_grid[2*x+2][2*y+2] = 0;
					break;
				case PLAYER_1:		// xx/x-
					led_grid[2*x+1][2*y+1] = 1;
					led_grid[2*x+2][2*y+1] = 1;
					led_grid[2*x+1][2*y+2] = 1;
					led_grid[2*x+2][2*y+2] = 0;
					break;
				case PLAYER_2:		// xx/-x
					led_grid[2*x+1][2*y+1] = 1;
					led_grid[2*x+2][2*y+1] = 1;
					led_grid[2*x+1][2*y+2] = 0;
					led_grid[2*x+2][2*y+2] = 1;
					break;
				case PLAYER_3:		// x-/xx
					led_grid[2*x+1][2*y+1] = 1;
					led_grid[2*x+2][2*y+1] = 0;
					led_grid[2*x+1][2*y+2] = 1;
					led_grid[2*x+2][2*y+2] = 1;
					break;
				case PLAYER_4:		// -x/xx
					led_grid[2*x+1][2*y+1] = 0;
					led_grid[2*x+2][2*y+1] = 1;
					led_grid[2*x+1][2*y+2] = 1;
					led_grid[2*x+2][2*y+2] = 1;
					break;
				case PLAYER_DYING:	// -x/x-
					led_grid[2*x+1][2*y+1] = 0;
					led_grid[2*x+2][2*y+1] = 1;
					led_grid[2*x+1][2*y+2] = 1;
					led_grid[2*x+2][2*y+2] = 0;
					break;
				case BOMB_A:		// x-/-x
					led_grid[2*x+1][2*y+1] = 1;
					led_grid[2*x+2][2*y+1] = 0;
					led_grid[2*x+1][2*y+2] = 0;
					led_grid[2*x+2][2*y+2] = 1;
					break;
				case BOMB_B:		// -x/x-
					led_grid[2*x+1][2*y+1] = 0;
					led_grid[2*x+2][2*y+1] = 1;
					led_grid[2*x+1][2*y+2] = 1;
					led_grid[2*x+2][2*y+2] = 0;
					break;
				case EXPLOSION_A:	// xx/--
					led_grid[2*x+1][2*y+1] = 1;
					led_grid[2*x+2][2*y+1] = 1;
					led_grid[2*x+1][2*y+2] = 0;
					led_grid[2*x+2][2*y+2] = 0;
					break;
				case EXPLOSION_B:	// --/xx
					led_grid[2*x+1][2*y+1] = 0;
					led_grid[2*x+2][2*y+1] = 0;
					led_grid[2*x+1][2*y+2] = 1;
					led_grid[2*x+2][2*y+2] = 1;
					break;
				case BLOCK:			// xx/xx
					led_grid[2*x+1][2*y+1] = 1;
					led_grid[2*x+2][2*y+1] = 1;
					led_grid[2*x+1][2*y+2] = 1;
					led_grid[2*x+2][2*y+2] = 1;
					break;
				default: //error?	// -x/--
					led_grid[2*x+1][2*y+1] = 0;
					led_grid[2*x+2][2*y+1] = 1;
					led_grid[2*x+1][2*y+2] = 0;
					led_grid[2*x+2][2*y+2] = 0;
					break;
			}
		}
	}
	
	// Set borders points
	// TODO - hardware cover (use coloured tape?)
	led_grid[0][0] = 1;
	led_grid[0][LED_HEIGHT - 1] = 1;
	led_grid[LED_WIDTH - 1][0] = 1;
	led_grid[LED_WIDTH - 1][LED_HEIGHT - 1] = 1;

#endif // TEST_LEDS
}

// Called by update_game, update LED lists for each monome
void update_monome_led_lists() {
	// Create a list of led sequences for each monome individually
	// We do this because cycling through all the LEDs leads to very dim displays
	// Note: each monome will be of different overall brightness. Oh well ; /
	
	int p;
	byte x, y; // byte so can be used directly for pins
	
	// Monome #1
	p = 0;
	for (y = 0; y < MONOME_PINS; y++) {
		for (x = 0; x < MONOME_PINS; x++) {
			if (led_grid[x][y]) {
				monome_1x[p] = x;
				monome_1y[p] = y;
				p++;
			}
		}
	}
	monome_leds_max[0] = p - 1;
	
	// Monome #2
	p = 0;
	for (y = MONOME_PINS; y < MONOME_PINS * 2; y++) {
		for (x = 0; x < MONOME_PINS; x++) {
			if (led_grid[x][y]) {
				monome_2x[p] = x;
				monome_2y[p] = y - MONOME_PINS;
				p++;
			}
		}
	}
	monome_leds_max[1] = p - 1;
	
	// Monome #3
	p = 0;
	for (y = 0; y < MONOME_PINS; y++) {
		for (x = MONOME_PINS; x < MONOME_PINS * 2; x++) {
			if (led_grid[x][y]) {
				monome_3x[p] = x - MONOME_PINS;
				monome_3y[p] = y;
				p++;
			}
		}
	}
	monome_leds_max[2] = p - 1;
	
	// Monome #4
	p = 0;
	for (y = MONOME_PINS; y < MONOME_PINS * 2; y++) {
		for (x = MONOME_PINS; x < MONOME_PINS * 2; x++) {
			if (led_grid[x][y]) {
				monome_4x[p] = x - MONOME_PINS;
				monome_4y[p] = y - MONOME_PINS;
				p++;
			}
		}
	}
	monome_leds_max[3] = p - 1;
}

// Called by main, update decoder pins
void update_display() {
	/*
	Monome boards layout
	Each monome has 8 pins, so we use a 3-to-8 decoder
	(0,0)	                   (15,0)
	    ------            ------
	    | #1 | PTAD[0-2]  | #2 | PTT[0-2]
	    ------            ------
	  PORTA[0-2]        PORTA[3-5]
	
	    ------            ------
	    | #3 | PTAD[3-5]  | #4 | PTT[3-5]
	    ------            ------
	  PORTB[0-2]        PORTB[3-5]
	(0,15)                     (15,15)
	*/
	
	int p1, p2, p3, p4;
	int c1, c2, c3, c4;
	int p, pmax;
	int delay;
	
	// Add all the monome maxes together so each cycles through at least once
	/*
	c1 = monome_leds_max[0];
	c2 = monome_leds_max[1];
	c3 = monome_leds_max[2];
	c4 = monome_leds_max[3];
	pmax = c1 + c2 + c3 + c4;
	// FIXME - change this into an infinite loop
	
	// For each cycle, update ports accordingly
	for (p = 0; p < pmax; p++) {
		p1 = (p1 < c1) ? p1 + 1 : 0;
		p2 = (p2 < c2) ? p2 + 1 : 0;
		p3 = (p3 < c3) ? p3 + 1 : 0;
		p4 = (p4 < c4) ? p4 + 1 : 0;
		PORTA = monome_1x[p1] | (monome_2x[p2] << 3); // #1 and #2 x values
		PORTB = monome_3x[p3] | (monome_4x[p4] << 3); // #3 and #4 x values
		PTAD  = monome_1y[p1] | (monome_2y[p2] << 3); // #1 and #2 y values
		PTT   = monome_3y[p3] | (monome_4y[p4] << 3); // #3 and #4 y values
		for (delay = LED_DELAY; delay > 0; delay--);
	}
	*/
	// Monome #1
	for (p = 0; p < monome_leds_max[0]; p++) {
		PORTA = (monome_1x[p]) | (monome_1y[p] << 3) | 0x00;
	}
	// Monome #2
	for (p = 0; p < monome_leds_max[1]; p++) {
		PORTA = (monome_2x[p]) | (monome_2y[p] << 3) | 0x40;
	}
	// Monome #3
	for (p = 0; p < monome_leds_max[2]; p++) {
		PORTA = (monome_3x[p]) | (monome_3y[p] << 3) | 0x80;
	}
	// Monome #4
	for (p = 0; p < monome_leds_max[3]; p++) {
		PORTA = (monome_4x[p]) | (monome_4y[p] << 3) | 0xC0;
	}
}

// For debugging, print out the grid
#ifdef TEST_PRINT
void print_led_grid() {
	byte x, y;
	printf("=========================\r\n");
	printf("led_grid at game_counter = %d\r\n", game_counter);
	for (y = 0; y < LED_HEIGHT; y++) {
		for (x = 0; x < LED_WIDTH; x++) {
			printf("%d ", led_grid[x][y]);
		}
		printf("\r\n");
	}
	printf("=========================\r\n");
}
#endif
