/*
 * Last updated: 2011/12/06
 * ~Philip Peng
 * 
 * This file is part of BomberNome.
 * 
 * BomberNome is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * BomberNome is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with BomberNome.	If not, see <http://www.gnu.org/licenses/>.
 */

#include "bombernome.h"

// Variables
// Game variables
unsigned int game_counter;
player_t *player_1, *player_2, *player_3, *player_4;
char game_grid[GRID_WIDTH][GRID_HEIGHT];

// Main
void main() {

	// Terminal
	TERMIO_Init();
#ifdef TEST_PRINT
	printf("=========================\r\n");
	printf("  Welcome to BomberNome\r\n");
	printf("=========================\r\n");
#endif
	
	// Setup everything
	setup_display();
	setup_input();
	setup_game();
	
	// Start things up
	game_counter = 0;
	update_game();
	EnableInterrupts;
	
	// Infinite loop
	for(;;) {
		update_display();
	}
}

// Standard Timer Overflow (clock)
void interrupt 16 timer_overflow(){
	// TFLG2 = [TOF][---][---][---][---][---][---][---]
	// TOF = Timer Overflow Flag (set to 1 to clear)
	TFLG2 &= 0x80;		// Clear TOF flag
	update_game();		// Update game
	
#ifdef TEST_INPUT
	if (game_counter % 8 == 0) { //Every ~2s
		simulate_input();
		update_game();
	}
#endif
#ifdef TEST_PRINT
	print_game_grid();	// Print out game state
	//print_led_grid();	// Print out display state
#endif
}

// Real Time Interrupt (input)
void interrupt 7 input_interrupt() {
	// CRGFLG = [RTIF][PORF][LVRF][LOCKIF][LOCK][TRACK][SCMIF][SCM]
	// RTIF = Real Time Interrupt Flag (set to 1 to clear)
	CRGFLG &= 0x80;		// Clear interrupt flag
	update_actions();	// Update player actions
}
